@@ -287,7 +287,7 @@ namespace memory
             DWORD FirmwareTableNameToUse = 0;
             for( i = 0; i < iBufferSizeNeeded; i++ )
             {
-                if( FirmwareTableEnumBuffer[i] > 0 && FirmwareTableEnumBuffer[i] <= offset )
+                if( FirmwareTableEnumBuffer[i] > 0 && FirmwareTableEnumBuffer[i] <= offset && offset <= FirmwareTableEnumBuffer[i] + 128 * 1024 )
                 {
                     FirmwareTableNameToUse = FirmwareTableEnumBuffer[i];
                 }
