@@ -50,7 +50,6 @@ const char *rbu_v1_pkt_size_file = "/sys/firmware/rbu/packetdatasize";
 
 const char *rbu_v2_fw_data_file = "/sys/class/firmware/dell_rbu/data";
 const char *rbu_v2_fw_load_file = "/sys/class/firmware/dell_rbu/loading";
-const char *rbu_v2_drv_data_file = "";
 const char *rbu_v2_img_type_file = "/sys/devices/platform/dell_rbu/image_type";
 const char *rbu_v2_pkt_size_file = "/sys/devices/platform/dell_rbu/packet_size";
 
@@ -118,8 +117,8 @@ const int RBU_PACKET_SIZE = 4096;
         while(!feof(hdr_fh))
         {
             ++pkt->pktNum;
-            memset(&(pkt->pktData), 0, bufSize - sizeof(rbu_packet));
-            fread(&(pkt->pktData), 1, bufSize - sizeof(rbu_packet), hdr_fh);
+            memset(&(pkt->pktData), 0, bufSize - sizeof(rbu_packet) + 1);
+            fread(&(pkt->pktData), 1, bufSize - sizeof(rbu_packet) + 1, hdr_fh);
             if (ferror(hdr_fh))
                 throw HdrFileIOErrorImpl(strerror(errno));
     
