@@ -41,8 +41,57 @@ using namespace std;
 
 namespace smbios
 {
+
     // validate the smbios table entry point
-    bool validateTableEntryPoint(
+    bool validateDMITableEntryPoint(
+        const smbiosLowlevel::dmi_table_entry_point *tempTEP,
+        bool strict,
+        ParseExceptionImpl &parseException
+        )
+    {
+        // This code checks for the following:
+        //       entry point structure checksum : As per the specs
+        //       anchor string : As per the specs
+        //
+        bool retval = true;
+
+        u8 checksum = 0;
+        const u8 *ptr = reinterpret_cast<const u8*>(tempTEP);
+        // don't overrun tempTEP if BIOS is buggy... (note sizeof() test here)
+        //      added especially to deal with buggy Intel BIOS.
+        for( unsigned int i = 0; i < sizeof(*tempTEP); ++i )
+        {
+            // stupid stuff to avoid MVC++ .NET runtime exception check for cast to different size
+            checksum = (checksum + ptr[i]) & 0xFF;
+        }
+
+        ostringstream oss;
+
+        DCERR("_DMI_ anchor: " << tempTEP->anchor[0] << tempTEP->anchor[1] << tempTEP->anchor[2] << tempTEP->anchor[3] << tempTEP->anchor[4] << tempTEP->anchor[5] << endl);
+        if(memcmp(tempTEP->anchor,"_DMI_",5)!=0) // Checking intermediate anchor string
+        {
+            oss << _("Intermediate anchor string does not match. anchor string: %(dmi_anchor)s") << endl;
+            retval = false;  // validation failed
+        }
+
+        DCERR("_DMI_ checksum: " << (int)checksum << endl);
+        if(checksum) // Checking entry point structure checksum
+        {
+            oss << _("Checksum check for table entry point should be zero. checksum: %(dmi_checksum)i ") << endl;
+            retval = false;  // validation failed
+        }
+
+        parseException.setParameter("dmi_anchor", reinterpret_cast<const char *>(tempTEP->anchor));
+        parseException.setParameter("dmi_checksum", static_cast<int>(checksum));
+
+        return retval;
+    }
+
+
+
+
+    // validate the smbios table entry point
+    bool validateSmbiosTableEntryPoint(
         const smbiosLowlevel::smbios_table_entry_point *tempTEP,
         bool strict,
         ParseExceptionImpl &parseException
@@ -83,35 +132,40 @@ namespace smbios
         ostringstream oss;
         oss << _("validation of table entry point failed") << endl;
 
-        if(memcmp(tempTEP->dmi_anchor,"_DMI_",5)!=0) // Checking intermediate anchor string
-        {
-            oss << _("Intermediate anchor string does not match. anchor string: %(dmi_anchor)s") << endl;
-            retval = false;  // validation failed
-        }
-        if(checksum!=0x00) // Checking entry point structure checksum
+        validateDMITableEntryPoint( &(tempTEP->dmi), strict, parseException );
+
+        DCERR("strict table checking: " << strict << endl);
+
+
+        DCERR("_SM_ checksum: " << (int)checksum << endl);
+
+        if(checksum) // Checking entry point structure checksum
         {
             oss << _("Checksum check for table entry point should be zero. checksum: %(checksum)i ") << endl;
             retval = false;  // validation failed
         }
+
+        DCERR("major_ver: " << (int)tempTEP->major_ver << endl);
         if(tempTEP->major_ver!=0x02)     // Checking smbios major version
         {
             oss << _("Major version of table entry point should be 2: %(major_version)i") << endl;
             retval = false;  // validation failed
         }
+
         // Entry Point Length field is at least 0x1f.
+        DCERR("eps_length: " << (int)tempTEP->eps_length << endl);
         if(tempTEP->eps_length < 0x0f)
         {
             oss << _("Entry Point Length field is at least 0x1f : %(eps_length)i") << endl;
             retval = false;  // validation failed
         }
 
-        parseException.setParameter("dmi_anchor", reinterpret_cast<const char *>(tempTEP->dmi_anchor));
         parseException.setParameter("checksum", static_cast<int>(checksum));
         parseException.setParameter("major_version", static_cast<int>(tempTEP->major_ver));
         parseException.setParameter("eps_length", static_cast<int>(tempTEP->eps_length));
         parseException.setMessageString(oss.str());
 
-        return strict ? retval : 1;
+        return retval;
     }
 
 
@@ -147,10 +201,10 @@ namespace smbios
         memory::IMemory *mem = memory::MemoryFactory::getFactory()->getSingleton();
 
         // new throws exception, no need to test.
-        u8 *newSmbiosBuffer = new u8[table_header.table_length];
+        u8 *newSmbiosBuffer = new u8[table_header.dmi.table_length];
         try
         {
-            mem->fillBuffer( newSmbiosBuffer, table_header.table_address, table_header.table_length );
+            mem->fillBuffer( newSmbiosBuffer, table_header.dmi.table_address, table_header.dmi.table_length );
 
             //delete old one, if necessary
             if( 0 != *smbiosBuffer )
@@ -183,11 +237,13 @@ namespace smbios
         ParseExceptionImpl parseException;
         if( offset )
         {
+            DCERR("SmbiosMemoryStrategy::getSmbiosTableHeader() using hardcoded offset: " << hex << offset << endl);
             parseException.setMessageString(_("SMBIOS Header not found at offset: %(offsetValue)i"));
             parseException.setParameter("offsetValue",offset);
         }
         else
         {
+            DCERR("SmbiosMemoryStrategy::getSmbiosTableHeader() Memory scan for smbios table." << endl);
             parseException.setMessageString(_("SMBIOS Header not found in search."));
         }
 
@@ -202,10 +258,33 @@ namespace smbios
             );
 
             // search for promising looking headers
-            if (memcmp (&tempTEP, "_SM_", 4) == 0)
+            // first, look for old-style DMI header
+            if (memcmp (&tempTEP, "_DMI_", 5) == 0)
             {
-                if(validateTableEntryPoint(&tempTEP, strict, parseException))
+                DCERR("Found _DMI_ anchor. Trying to parse legacy DMI structure." << endl);
+                dmi_table_entry_point *dmiTEP = reinterpret_cast<dmi_table_entry_point *>(&tempTEP);
+                memmove(&(tempTEP.dmi), &dmiTEP, sizeof(dmi_table_entry_point));
+                // fake the rest of the smbios table entry point...
+                tempTEP.major_ver=2;
+                tempTEP.minor_ver=0;
+                if(validateDMITableEntryPoint(dmiTEP, strict, parseException))
                 {
+                    DCERR("Found valid _DMI_ entry point at offset: " << fp << endl);
+                    break;
+                }
+            }
+
+            // then, look for new-style smbios header. This will always
+            // occur before _DMI_ in memory
+            if (offset || (memcmp (&tempTEP, "_SM_", 4) == 0))
+            {
+                // if we are passed a hardcoded offset (EFI?), it is possible
+                // that machine doesnt have _SM_ anchor, but still has valid
+                // table. Just try validating the table and skip _SM_ check.
+                DCERR("Found _SM_ anchor or using hardcoded offset. Trying to parse Smbios Entry Point." << endl);
+                if(validateSmbiosTableEntryPoint(&tempTEP, strict, parseException))
+                {
+                    DCERR("Found valid _SM_ entry point at offset: " << fp << endl);
                     break;
                 }
             }
