@@ -25,12 +25,14 @@
 namespace smi
 {
     // define our exceptions
+    DEFINE_EXCEPTION_EX( SmiExceptionImpl, smi, SmiException);
     DEFINE_EXCEPTION_EX( InvalidSmiModeImpl, smi, InvalidSmiMode);
     DEFINE_EXCEPTION_EX( ParameterErrorImpl, smi, ParameterError);
     DEFINE_EXCEPTION_EX( UnsupportedSmiImpl, smi, UnsupportedSmi);
     DEFINE_EXCEPTION_EX( UnhandledSmiImpl,   smi, UnhandledSmi);
     DEFINE_EXCEPTION_EX( SmiExecutedWithErrorImpl, smi, SmiExecutedWithError);
     DEFINE_EXCEPTION_EX( PasswordVerificationFailedImpl, smi, PasswordVerificationFailed);
+    DEFINE_EXCEPTION_EX( ConfigErrorImpl, smi, ConfigError);
 
     class SmiStrategy
     {
